@@ -20,3 +20,7 @@
 from gettext import install
 
 install("debomatic")
+
+
+class dom:
+    pass
