@@ -20,27 +20,84 @@
 
 import os
 from ast import literal_eval
+from contextlib import contextmanager
 from logging import debug, error, info
 from re import findall, search, sub
-from shutil import copymode, move
-from subprocess import call, check_output
+from shutil import copy, copymode, move
+from subprocess import Popen, check_output
 from tempfile import NamedTemporaryFile
+from threading import Semaphore
 from time import strftime
 from urllib.request import Request, urlopen
 from urllib.error import HTTPError, URLError
 
+from Debomatic import dom
 from .gpg import GPG
 from .modules import Module
 from .exceptions import DebomaticError
 
 
+class BuildTask:
+
+    def __init__(self, build, package, version, distribution, queue):
+        self._build = build
+        self._package = package
+        self._version = version
+        self._distribution = distribution
+        self._queue = queue
+        self._pid = 0
+
+    def __enter__(self):
+        for task in self._queue:
+            if (self._package == task._package and
+               self._version == task._version and
+               self._distribution == task._distribution):
+                info(_('Build already scheduled for '
+                       'package %(package)s_%(version)s in %(dist)s') %
+                     {'package': self._package, 'version': self._version,
+                      'dist': self._distribution})
+                self._skip_removal()
+                raise DebomaticError
+        self._queue.append(self)
+        return self
+
+    def __exit__(self, type, value, tb):
+        if self in self._queue:
+            self._queue.remove(self)
+
+    def _skip_removal(self):
+        for pkgfile in set(self._build.files):
+            if os.path.isfile(pkgfile):
+                for build in [x._build for x in self._queue
+                              if x._build != self]:
+                    if pkgfile in build.files:
+                        self._build.files.remove(pkgfile)
+                        debug(_('Skipping removal of file %s') % pkgfile)
+
+    def get_pid(self):
+        return self._pid
+
+    @contextmanager
+    def set_pid(self, pid):
+        self._pid = pid
+        try:
+            yield
+        finally:
+            self._pid = 0
+
+    def match(self, package, version, distribution):
+        if (self._package == package and
+           self._version == version and
+           self._distribution == distribution):
+            return self
+
+
 class Build:
 
-    def __init__(self, opts, dists, changesfile=None, package=None,
-                 distribution=None, binnmu=None, extrabd=None, maintainer=None,
-                 origin=None, uploader=None):
-        self.opts = opts
-        self.dists = dists
+    def __init__(self, changesfile=None, package=None, distribution=None,
+                 binnmu=None, extrabd=None, maintainer=None, origin=None,
+                 uploader=None):
+        self.buildtask = None
         self.changesfile = changesfile
         self.package = package
         self.distribution = distribution
@@ -50,21 +107,33 @@ class Build:
         self.origin = origin
         self.uploader = uploader
         self.files = set()
-        self.incoming = self.opts.get('debomatic', 'incoming')
+        self.incoming = dom.opts.get('debomatic', 'incoming')
+        self.hostarchitecture = None
 
     def _build(self):
         self._parse_distribution()
-        if self.distribution in self.opts.get('distributions', 'blacklist'):
+        if self.distribution in dom.opts.get('distributions', 'blacklist'):
             self._remove_files()
             error(_('Distribution %s is disabled') % self.distribution)
             raise DebomaticError
+        if dom.opts.has_section('crossbuild'):
+            if dom.opts.get('crossbuild', 'crossbuild'):
+                self.hostarchitecture = dom.opts.get('crossbuild',
+                                                     'hostarchitecture')
         try:
-            self._fetch_files()
-            self._setup_chroot()
+            if self.changesfile:
+                package = os.path.basename(self.changesfile).split('_')[0]
+                version = os.path.basename(self.changesfile).split('_')[1]
+            else:
+                package, version = self.package
+            with BuildTask(self, package, version, self.distribution,
+                           dom.buildqueue) as bt:
+                self.buildtask = bt
+                self._fetch_files()
+                self._setup_chroot()
+                self._build_package()
         except DebomaticError:
             self._remove_files()
-        else:
-            self._build_package()
 
     def _build_package(self):
         uploader_email = ''
@@ -74,25 +143,28 @@ class Build:
             os.mkdir(builddir)
         if self.uploader:
             uploader_email = self.uploader[1].decode('utf-8')
-        architecture = self.opts.get('debomatic', 'architecture')
+        architecture = dom.opts.get('debomatic', 'architecture')
         if architecture == 'system':
             b_arch = check_output(['dpkg-architecture', '-qDEB_BUILD_ARCH'])
             architecture = b_arch.strip().decode('utf-8')
-        mod = Module(self.opts)
+        mod = Module()
         mod.args.architecture = architecture
         mod.args.directory = self.buildpath
         mod.args.distribution = self.distribution
-        mod.args.dists = self.dists
+        mod.args.dists = dom.dists
         mod.args.dsc = self.dscfile
         mod.args.files = self.files
         mod.args.package = packageversion
         mod.args.uploader = uploader_email
+        mod.args.hostarchitecture = self.hostarchitecture
         mod.execute_hook('pre_build')
         info(_('Building %s') % os.path.basename(self.dscfile))
         command = ['sbuild', '-A', '-s', '-d', self.distribution,
                    '--arch=%s' % architecture, '-c',
                    '%s-%s-debomatic' % (self.distribution, architecture),
                    self.dscfile]
+        if self.hostarchitecture:
+            command.insert(6, '--host=%s' % self.hostarchitecture)
         if self.changesfile:
             with open(self.upload, 'r') as fd:
                 data = fd.read()
@@ -105,7 +177,7 @@ class Build:
                                findall(' \S+ \((\S+)\) \S+; ', data)[-1])
             except IndexError:
                 pass
-        suite = self.dists.get(self.distribution, 'suite')
+        suite = dom.dists.get(self.distribution, 'suite')
         if not (self.distribution == suite or
                 self.distribution.startswith(suite)):
             command.insert(-1, '--build-dep-resolver=aptitude')
@@ -116,9 +188,13 @@ class Build:
                                             architecture)
         else:
             buildlog = '%s_%s.build' % (packageversion, architecture)
+        if self.hostarchitecture:
+            buildlog = sub('(.*_)\S+(\.build)',
+                           '\\1%s\\2' % self.hostarchitecture, buildlog)
         if self.extrabd:
             for extrabd in self.extrabd:
                 command.insert(-1, '--add-depends=%s' % extrabd)
+                command.insert(-1, '--build-dep-resolver=aptitude')
         if self.maintainer:
             command.remove('-A')
             command.remove('-s')
@@ -134,6 +210,10 @@ class Build:
                         pass
         for sbuildcommand in self._commands():
             command.insert(-1, sbuildcommand)
+        if self.hostarchitecture:
+            command.insert(-1, '--chroot-setup-commands=%s' %
+                           ('dpkg --add-architecture %s' %
+                            self.hostarchitecture))
         with open(os.devnull, 'w') as fd:
             try:
                 os.chdir(os.path.join(self.buildpath, 'pool', packageversion))
@@ -141,11 +221,12 @@ class Build:
                 if os.path.exists(buildlink):
                     os.unlink(buildlink)
                 os.symlink(buildlog, buildlink)
-                result = call(command, stdout=fd, stderr=fd)
+                process = Popen(command, stdout=fd, stderr=fd)
+                with self.buildtask.set_pid(process.pid):
+                    process.wait()
             except OSError:
                 error(_('Invoication of sbuild failed'))
-                result = -1
-        if result == 0:
+        if process.returncode == 0:
             mod.args.success = True
         mod.execute_hook('post_build')
         self._remove_files()
@@ -156,7 +237,7 @@ class Build:
         types = ('pre-build-commands', 'chroot-setup-commands',
                  'starting-build-commands', 'finished-build-commands',
                  'chroot-cleanup-commands', 'post-build-commands')
-        commandsdir = self.opts.get('chroots', 'commands')
+        commandsdir = dom.opts.get('chroots', 'commands')
         if os.path.isdir(commandsdir):
             for type in types:
                 if os.path.isdir(os.path.join(commandsdir, type)):
@@ -184,7 +265,7 @@ class Build:
                 pass
 
         dscfile = None
-        components = self.dists.get(self.origin, 'components').split()
+        components = dom.dists.get(self.origin, 'components').split()
         if self.changesfile:
             package = os.path.basename(self.changesfile).split('_')[0]
             for filename in self.files:
@@ -200,14 +281,14 @@ class Build:
             package = self.package[0]
             version = sub('^\d+\:', '', self.package[1])
             dscfile = '%s_%s.dsc' % (package, version)
-            if not self.dists.has_section(self.origin):
+            if not dom.dists.has_section(self.origin):
                 error(_('Distribution %s not configured') % self.distribution)
                 raise DebomaticError
             self.dscfile = os.path.join(self.incoming, dscfile)
             if not os.path.isfile(self.dscfile):
                 debug(_('Downloading missing %s') % dscfile)
                 for component in components:
-                    _download_files(self.dists.get(self.origin, 'mirror'),
+                    _download_files(dom.dists.get(self.origin, 'mirror'),
                                     component, package,
                                     dscfile, self.dscfile)
                     if os.path.isfile(self.dscfile):
@@ -223,9 +304,9 @@ class Build:
         for entry in findall('\s\w{32}\s\d+\s(\S+)', data):
             if not os.path.isfile(os.path.join(self.incoming, entry)):
                 debug(_('Downloading missing %s') % entry)
-                for component in self.dists.get(self.origin,
-                                                'components').split():
-                    _download_files(self.dists.get(self.origin, 'mirror'),
+                for component in dom.dists.get(self.origin,
+                                               'components').split():
+                    _download_files(dom.dists.get(self.origin, 'mirror'),
                                     component, package, entry,
                                     os.path.join(self.incoming, entry))
                     if os.path.isfile(os.path.join(self.incoming, entry)):
@@ -239,10 +320,15 @@ class Build:
                 error(_('Unable to fetch %s') % entry)
                 raise DebomaticError
 
+    def _lock_chroot(self, chrootname):
+        if chrootname not in dom.chroots:
+            dom.chroots[chrootname] = Semaphore()
+        dom.chroots[chrootname].acquire()
+
     def _map_distribution(self):
-        if self.opts.has_option('distributions', 'mapper'):
+        if dom.opts.has_option('distributions', 'mapper'):
             try:
-                mapper = literal_eval(self.opts.get('distributions', 'mapper'))
+                mapper = literal_eval(dom.opts.get('distributions', 'mapper'))
             except SyntaxError:
                 pass
             else:
@@ -267,11 +353,11 @@ class Build:
                 error(_('Bad .changes file: %s') % self.upload)
                 raise DebomaticError
         self._map_distribution()
-        if not self.dists.has_section(self.distribution):
+        if not dom.dists.has_section(self.distribution):
             error(_('Distribution %s not configured') % self.distribution)
             raise DebomaticError
         if self.origin:
-            if not self.dists.has_section(self.origin):
+            if not dom.dists.has_section(self.origin):
                 error(_('Distribution %s not configured') % self.distribution)
                 raise DebomaticError
         else:
@@ -288,36 +374,37 @@ class Build:
         self.buildpath = os.path.join(self.incoming, self.distribution)
         if not os.path.exists(os.path.join(self.buildpath)):
             os.mkdir(os.path.join(self.buildpath))
-        architecture = self.opts.get('debomatic', 'architecture')
+        architecture = dom.opts.get('debomatic', 'architecture')
         if architecture == 'system':
             b_arch = check_output(['dpkg-architecture', '-qDEB_BUILD_ARCH'])
             architecture = b_arch.strip().decode('utf-8')
-        debootstrap = self.opts.get('debomatic', 'debootstrap')
+        debootstrap = dom.opts.get('debomatic', 'debootstrap')
+        chrootname = '%s-%s-debomatic' % (self.distribution, architecture)
+        self._lock_chroot(chrootname)
         with open(os.devnull, 'w') as fd:
             chroots = check_output(['schroot', '-l'], stderr=fd)
-        if not search('chroot:%s-%s-debomatic' % (self.distribution,
-                                                  architecture),
-                      chroots.decode()):
+        if not search('chroot:%s' % chrootname, chroots.decode()):
             action = 'create'
-        mod = Module(self.opts)
+        mod = Module()
         mod.args.architecture = architecture
         mod.args.action = action
         mod.args.directory = self.buildpath
         mod.args.distribution = self.distribution
-        mod.args.dists = self.dists
+        mod.args.dists = dom.dists
         mod.execute_hook('pre_chroot')
         for d in ('logs', 'pool'):
             if not os.path.exists(os.path.join(self.buildpath, d)):
                 os.mkdir(os.path.join(self.buildpath, d))
         if action:
-            profile = self.opts.get('chroots', 'profile')
+            profile = dom.opts.get('chroots', 'profile')
             if not os.path.isdir(os.path.join('/etc/schroot', profile)):
                 error(_('schroot profile %s not found') % profile)
+                self._unlock_chroot(chrootname)
                 raise DebomaticError
             logfile = ('%s/logs/%s.%s' %
                        (self.buildpath, self.distribution,
                         strftime('%Y%m%d_%H%M%S')))
-            target = self.dists.get(self.distribution, 'suite')
+            target = dom.dists.get(self.distribution, 'suite')
             if target == self.distribution:
                 pattern = '%s-%s-debomatic' % (self.distribution, architecture)
             else:
@@ -325,39 +412,50 @@ class Build:
                                                   self.distribution)
             with open(logfile, 'w') as fd:
                 try:
-                    debug(_('Creating chroot %s') % pattern)
-                    components = ','.join(self.dists.get(self.distribution,
-                                                         'components').split())
+                    debug(_('Creating chroot %(dist)s-%(arch)s-debomatic') %
+                          {'dist': self.distribution, 'arch': architecture})
+                    components = ','.join(dom.dists.get(self.distribution,
+                                                        'components').split())
                     command = ['sbuild-createchroot',
                                '--arch=%s' % architecture,
                                '--chroot-suffix=-debomatic',
                                '--debootstrap=%s' % debootstrap,
                                '--components=%s' % components, target,
                                os.path.join(self.buildpath, self.distribution),
-                               self.dists.get(self.distribution, 'mirror')]
+                               dom.dists.get(self.distribution, 'mirror')]
                     if target != self.distribution:
                         command[2] = ('--chroot-suffix=-%s-debomatic' %
                                       self.distribution)
-                    if self.dists.has_option(self.distribution,
-                                             'extrapackages'):
-                        packages = self.dists.get(self.distribution,
-                                                  'extrapackages').split()
+                    if dom.dists.has_option(self.distribution,
+                                            'extrapackages'):
+                        packages = dom.dists.get(self.distribution,
+                                                 'extrapackages').split()
                         command.insert(-3, '--include=%s' % ','.join(packages))
                         packages = '--include=%s' % ','.join(packages)
-                    if call(command, stdout=fd, stderr=fd):
+                    process = Popen(command, stdout=fd, stderr=fd)
+                    with self.buildtask.set_pid(process.pid):
+                        process.wait()
+                    if process.returncode:
                         error(_('Failed creating %(dist)s-%(arch)s-debomatic')
                               % {'dist': self.distribution,
                                  'arch': architecture})
                         mod.execute_hook('post_chroot')
+                        self._unlock_chroot(chrootname)
                         raise DebomaticError
                 except OSError:
                     error(_('Unable to launch sbuild-createchroot'))
                     mod.execute_hook('post_chroot')
+                    self._unlock_chroot(chrootname)
                     raise DebomaticError
-            if self.dists.has_option(self.distribution, 'extramirrors'):
+            if dom.dists.has_option(self.distribution, 'extramirrors'):
                 with open(os.path.join(self.buildpath, self.distribution,
                                        'etc/apt/sources.list'), 'a') as fd:
-                    fd.write(self.dists.get(self.distribution, 'extramirrors'))
+                    fd.write(dom.dists.get(self.distribution, 'extramirrors'))
+            if dom.opts.has_option('repository', 'pubring'):
+                if os.path.isfile(dom.opts.get('repository', 'pubring')):
+                    copy(dom.opts.get('repository', 'pubring'),
+                         os.path.join(self.buildpath, self.distribution,
+                                      'etc/apt/trusted.gpg.d/debomatic.gpg'))
             chroots = '/etc/schroot/chroot.d'
             for file in os.listdir(chroots):
                 if file.startswith(pattern):
@@ -380,6 +478,11 @@ class Build:
                     move(tmp.name, fd.name)
             mod.args.success = True
             mod.execute_hook('post_chroot')
+        self._unlock_chroot(chrootname)
+
+    def _unlock_chroot(self, chrootname):
+        if chrootname in dom.chroots:
+            dom.chroots[chrootname].release()
 
     def run(self):
         if self.changesfile:
@@ -402,7 +505,7 @@ class Build:
                 error(_('Bad .changes file: %s') % self.upload)
                 raise DebomaticError
             try:
-                with GPG(self.opts, self.upload) as gpg:
+                with GPG(self.upload) as gpg:
                     try:
                         self.uploader = gpg.check()
                     except DebomaticError:
