@@ -27,6 +27,7 @@ from signal import signal, SIGINT, SIGTERM
 from sys import stdin, stdout, stderr
 from time import sleep
 
+from Debomatic import dom
 from .exceptions import DebomaticError
 
 
@@ -85,9 +86,16 @@ class Process:
                 self.fd.close()
             raise ex
 
+    def _notify_systemd(self):
+        try:
+            import systemd.daemon
+            systemd.daemon.notify('READY=1')
+        except (ImportError, SystemError):
+            pass
+
     def _quit(self, signum=None, frame=None):
         info(_('Waiting for threads to complete...'))
-        self.pool.shutdown()
+        dom.pool.shutdown()
         self.mod_sys.execute_hook('on_quit')
         self._unlock()
         os.unlink(self.pidfile)
@@ -140,6 +148,7 @@ class Process:
         signal(SIGTERM, self._quit)
         if self.daemonize:
             self._daemonize()
+        self._notify_systemd()
         self.launcher()
 
 
