@@ -34,6 +34,8 @@ class DebomaticModule_Piuparts:
             return
         if not os.access(self.piuparts, os.X_OK):
             return
+        if args.hostarchitecture:
+            return
         if args.opts.has_section('piuparts'):
             options = args.opts.get('piuparts', 'options').strip().split()
         else:
@@ -47,8 +49,8 @@ class DebomaticModule_Piuparts:
                 log = os.path.join(resultdir, args.package) + '.piuparts'
                 with open(log, 'a') as fd:
                     cmd = [self.piuparts,
-                           '-I', 'sys/fs/aufs/*',
-                           '-I', 'build/*',
+                           '-I', '/sys/fs/aufs/.*',
+                           '-I', '/build/.*',
                            '-d', '%s' % distribution,
                            '-D', '%s' % mirror.split('/')[-1],
                            '--schroot=chroot:%s' % schroot,
